#include "extra_menu.hpp"

#include "renderer.hpp"
#include "common.hpp"
#include "themes.hpp"
#include "graphics.hpp"
#include "config.hpp"
#include "global.hpp"
#include "updater.hpp"

#include "busy_indicator.hpp"
#include "shutdown.hpp"

ExtraMenuItem::ExtraMenuItem(const std::string c, const std::string& s, TimeFv *cb)
  : command(c), shortcut(s), callback(cb)
{}

void ExtraMenu::add_item(const ExtraMenuItem& e)
{
  items.push_back(std::make_pair(e, false));
}

void ExtraMenu::add_persistent_item(const ExtraMenuItem& e)
{
  items.push_back(std::make_pair(e, true));
}

ExtraMenu::ExtraMenu(const std::string& h, bool dd)
  : header(h), drop_down(dd), pos(0), o(Overlay("extra menu"))
{}

ExtraMenu::~ExtraMenu()
{
  for (std::vector<std::pair<ExtraMenuItem, bool> >::const_iterator iter = items.begin();
       iter != items.end(); ++iter)
    if (!iter->second)
      delete iter->first.callback;
}

void ExtraMenu::set_drop_down_pos(int x_pos, int y_pos)
{
  dd_x = x_pos;
  dd_y = y_pos;
}

void ExtraMenu::cleanup()
{
  Render *render = S_Render::get_instance();
  
  render->wait_and_aquire();
  o.cleanup();
  render->image_mut.leaveMutex();
}

void ExtraMenu::mainloop()
{
  InputMaster *im = S_InputMaster::get_instance();
  Global *global = S_Global::get_instance();
  Config *conf = S_Config::get_instance();
  ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();

  conf->s_sam(true);

  BusyIndicator &busy_indicator = BusyIndicator::get_instance();
  Shutdown *sd = S_Shutdown::get_instance();

  busy_indicator.idle();

  bool audio_fullscreen_was_running = true;

  if (!screen_updater->timer.status("audio_fullscreen"))
    audio_fullscreen_was_running = false;
  else
    screen_updater->timer.run_and_deactivate("audio_fullscreen");

  Input input;
  
  bool update_needed = true;

  while (!(input.command == "back" && input.mode == "general") &&
	 !(input.command == "second_action" && input.mode == "general"))
    {
      if (update_needed)
	print();

      input = im->device->get_input();
      
      if (sd->is_enabled()) {
      	sd->cancel();
      	continue;
      }

      update_needed = true;

      if (input.command == "prev")
	{
	  if (pos != 0)
	    --pos;
	  else
	    pos = items.size()-1;
 	}
      else if (input.command == "next")
 	{
	  if (pos != items.size()-1)
	    ++pos;
	  else
	    pos = 0;
 	}
      else if (input.command == "action")
	{
	  busy_indicator.busy_no_print();

	  // FIXME: evil hack
	  if (vector_lookup(items, pos).first.command != gettext("Zoom"))
	    cleanup();

	  if (audio_fullscreen_was_running)
	    screen_updater->timer.activate("audio_fullscreen");

	  vector_lookup(items, pos).first.callback->call();

	  // FIXME: evil hack
	  if (vector_lookup(items, pos).first.command == gettext("Zoom"))
	    cleanup();

	  // FIXME: audio fullscreen?

	  conf->s_sam(false);

	  return;
	}
      else
	update_needed = !global->check_commands(input, true);
    }

  cleanup();

  if (audio_fullscreen_was_running)
    screen_updater->timer.activate("audio_fullscreen");

  conf->s_sam(false);
}

void ExtraMenu::print()
{
  Render *render = S_Render::get_instance();
  Themes *themes = S_Themes::get_instance();
  Config *conf = S_Config::get_instance();

  render->wait_and_aquire();

  if (o.elements.size() > 0)
    o.partial_cleanup();

  int max_x = conf->p_v_res()-250, max_y = 30;  
  int box_height = max_y*items.size()+30+35;

  int x = (conf->p_v_res()-(max_x+30))/2;
  int y = (conf->p_h_res()-(box_height))/2;

  if (drop_down) {
    x = dd_x;
    y = dd_y;
    box_height = max_y*items.size()+30;
    max_x = 0;
    for (std::vector<std::pair<ExtraMenuItem, bool> >::const_iterator iter = items.begin();
	 iter != items.end(); ++iter)
      max_x = std::max(max_x, string_format::calculate_string_width(iter->first.command, "Vera/17"));
  }
  
  o.add(new RObj(x-15+2, y-15+2, max_x+30+1, box_height+1, 0, 0, 0, 200, 0));
  o.add(new RObj(x-15, y-15, max_x+30, box_height,  
		 themes->dialog_background1, themes->dialog_background2, 
		 themes->dialog_background3, themes->dialog_background4, 1));

  int x_size;
  
  if (!header.empty()) {
    x_size = string_format::calculate_string_width(header, "Vera/20");
    
    o.add(new TObj(header, "Vera/20", (conf->p_v_res()-x_size)/2, y-5, 
		   themes->dialog_font1, themes->dialog_font2, themes->dialog_font3, 3));
    y += 35;
  }
  
  if (pos == -1) 
    o.add(new RObj(x-5, y-3, max_x+10, 25+6, 0, 0, 0, 225, 2));

  int i = 0;

  for (std::vector<std::pair<ExtraMenuItem, bool> >::const_iterator iter = items.begin();
       iter != items.end(); ++iter, ++i) {
    if (i == pos)
      o.add(new RObj(x-5, y-3, max_x+10, 25+6, themes->dialog_marked1, 
		     themes->dialog_marked2, themes->dialog_marked3, 
		     themes->dialog_marked4, 2));

    o.add(new TObj(iter->first.command, "Vera/17", x, y, themes->dialog_font1, 
		   themes->dialog_font2, themes->dialog_font3, 3));
    x_size = string_format::calculate_string_width(iter->first.shortcut, "Vera/17");
    o.add(new TObj(iter->first.shortcut, "Vera/17", x + max_x - x_size - 5, y, 
		   themes->dialog_font1, themes->dialog_font2, 
		   themes->dialog_font3, 3));
    y += 30;
  }

  render->draw_and_release("extra menu screen", true);
}
