#include "movie_db.hpp"

#include "common-feature.hpp"

#include "graphics.hpp"
#include "updater.hpp"
#include "busy_indicator.hpp"

// ostringstream
#include <sstream>

#include <vector>
#include <string>

#include <iostream>
#include <fstream>

using std::vector;
using std::string;
using std::list;

// FIXME: always store db in homedir?
MovieDB::MovieDB(const string& filename, bool p)
  : db((conf->p_var_data_dir() + filename).c_str()), physical(p)
{
  if (conf->p_v_res() == 720)
    y_start = 70;
  else if (conf->p_v_res() == 800)
    y_start = 90;
  else if (conf->p_v_res() == 1024)
    y_start = 110;
  else if (conf->p_v_res() == 1280)
    y_start = 110;
  else
    y_start = 70; // default value

  image_width = 115;
  image_height = 120;

  if (conf->p_v_res() == 720)
    image_height = 105;

  if (conf->p_h_res() == 405) {
    image_width = 65;
    image_height = 67;
  }

  image_width_all = image_width+25;
  image_height_all = image_height+55;
  image_height_all_eks_text = image_height + 6;

  images_per_row = (conf->p_v_res()-2*25)/image_width_all;

  // lets make some more room for text 
  image_width_all += ((conf->p_v_res()-2*25)%image_width_all)/images_per_row;

  rows = (conf->p_h_res()-2*y_start+20)/image_height_all;
  rows_search = (conf->p_h_res()-2*y_start+20-40)/image_height_all;

  image_height_all = (conf->p_h_res()-2*y_start+20)/rows;
  image_height_all_search = (conf->p_h_res()-2*y_start+20-40)/rows_search;

  if (!db)
    DebugPrint perror(gettext("Movie database could not be opened or created"),
		      Print::DEBUGGING, DebugPrint::INFO, "MovieDB");

  imdb_dir = "/var/lib/mms/movies/";
  if (conf->p_homedir() != "/etc/mms/")
    imdb_dir = conf->p_homedir() + "movies/";

  if (!file_exists(imdb_dir)) {
    if (mkdir(imdb_dir.c_str(), 0777) == -1)
      DebugPrint perror(gettext("Could not create directory ") + imdb_dir,
                       Print::DEBUGGING, DebugPrint::CRITICAL, "MOVIE");
  }

  check_db();
}
  
void MovieDB::create_db()
{
  vector<string> fields;
  fields.push_back("version INTEGER NOT NULL");
  db.createTable("Schema", fields);

  db.execute("INSERT INTO Schema VALUES('1')");

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("parent INTEGER");
  fields.push_back("filename TEXT");
  fields.push_back("name TEXT");
  fields.push_back("is_folder INTEGER"); // 1 = true
  db.createTable("Folders", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("name TEXT");
  db.createTable("Directors", fields);
  
  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("did INTEGER NOT NULL");
  fields.push_back("mid INTEGER NOT NULL");
  db.createTable("DMovie", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("name TEXT");
  fields.push_back("role TEXT");
  db.createTable("WritingCredits", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("wcid INTEGER NOT NULL");
  fields.push_back("mid INTEGER NOT NULL");
  db.createTable("WCMovie", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("name TEXT");
  db.createTable("Genre", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("gid INTEGER NOT NULL");
  fields.push_back("mid INTEGER NOT NULL");
  db.createTable("GMovie", fields);
  
  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("name TEXT");
  fields.push_back("role TEXT");
  db.createTable("Actors", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("aid INTEGER NOT NULL");
  fields.push_back("mid INTEGER NOT NULL");
  db.createTable("AMovie", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("title TEXT");

  if (physical)
    fields.push_back("path TEXT");

  fields.push_back("url TEXT");
  fields.push_back("runtime TEXT");
  fields.push_back("tagline TEXT");
  fields.push_back("plot TEXT");
  fields.push_back("rating REAL");
  fields.push_back("votes INTEGER");
  fields.push_back("year INTEGER");
  fields.push_back("top250 INTEGER");

  if (physical)
    db.createTable("HDMovie", fields);
  else
    db.createTable("Movie", fields);
}

void MovieDB::check_db()
{
  if (db.hasTable("Schema")) {
    // check for correct version
    SQLQuery *q = db.query("Schema", "SELECT version FROM %t");
    if (q && q->numberOfTuples() > 0) {
      SQLRow &row = (*q)[0];
      if (row["version"] != "1") {
	// FIXME: better to upgrade db
	db.execute("DROP TABLE Schema");
	db.execute("DROP TABLE Directors");
	db.execute("DROP TABLE DMovie");
	db.execute("DROP TABLE WritingCredits");
	db.execute("DROP TABLE WMovie");
	db.execute("DROP TABLE Genre");
	db.execute("DROP TABLE GMovie");
	db.execute("DROP TABLE Actors");
	db.execute("DROP TABLE AMovie");
	db.execute("DROP TABLE HDMovie");
	db.execute("DROP TABLE Movie");
	create_db();
      }
    }
    delete q;
  } else {
    // create the whole db
    create_db();
  }
}

void MovieDB::reget_movie_information()
{
  CIMDBMovie m = vector_lookup(files, folders.top().second);
  imdb_search_mainloop(m.name, m, false);
  files[folders.top().second] = m;
}

void MovieDB::print_info()
{
  BusyIndicator::get_instance().idle();

  if (vector_lookup(files, folders.top().second).m_strTitle.empty()) {
    DialogWaitPrint pdialog(2000);
    pdialog.add_line(gettext("No IMDb information found about movie"));
    pdialog.add_line(gettext("Please update information"));
    pdialog.print();
  } else
    print_information(vector_lookup(files, folders.top().second));
}

void MovieDB::search_func()
{
  BusyIndicator::get_instance().idle();

  ConstFunctor<MovieDB, std::string, vector<std::pair<string, int> > > gen_list_func(this, &MovieDB::gen_search_list);
  ConstFunctor<MovieDB, vector<CIMDBMovie>, void> print_func(this, &MovieDB::print);

  int pos = input_master->device->graphical_search(files, &print_func, &gen_list_func,
						   search_mode, search_str, lowercase_search_str, 
						   offset, search_help_offset);

  if (pos != -1)
    folders.top().second = pos;
}

void MovieDB::search_imdb()
{
  CIMDBMovie m;
  if (imdb_search_mainloop("", m, true))
    files.push_back(m);

  // reset map
  input_master->set_map("movie");
}

void MovieDB::prev()
{
  if (files.size() > images_per_row) {
    for (int i = 0; i < images_per_row; ++i) {
      if (folders.top().second == 0) {
	folders.top().second = files.size()-1;
	i += images_per_row-files.size()%images_per_row;
      }
      else 
	folders.top().second -= 1;
    }
  }
}

void MovieDB::next()
{
  if (files.size() > images_per_row) {
    for (int i = 0; i < images_per_row; ++i) {
      if (folders.top().second == files.size()-1 && i == 0) {
	folders.top().second = 0;
	break;
      } else if (folders.top().second == files.size()-1) {
	break;
      } else {
	folders.top().second += 1;
      }
    }
  }
}

void MovieDB::left()
{
  if (folders.top().second != 0)
    folders.top().second -= 1;
  else
    folders.top().second = files.size() - 1;
}

void MovieDB::right()
{
  folders.top().second = (folders.top().second+1)%files.size();
}

void MovieDB::page_up()
{
  int jump = conf->p_jump();

  if (search_mode)
    jump = images_per_row*rows_search;
  else
    jump = images_per_row*rows;

  if (files.size() > jump) {
    int diff = folders.top().second-jump;
    if (folders.top().second == 0)
      folders.top().second = (files.size()-1)+diff;
    else if (diff < 0)
      folders.top().second = 0;
    else
      folders.top().second = diff;
  }
}

void MovieDB::page_down()
{
  int jump = conf->p_jump();

  if (search_mode)
    jump = images_per_row*rows_search;
  else
    jump = images_per_row*rows;

  if (files.size() > jump) {
    if (folders.top().second > (files.size() - jump) && folders.top().second != (files.size()-1))
      folders.top().second = files.size()-1;
    else
      folders.top().second = (folders.top().second+jump)%files.size();
  }
}

void MovieDB::print_information(const CIMDBMovie& m)
{
  render->prepare_new_image();

  render->current.add(new PObj(themes->movie_background, 0, 0, 0, SCALE_FULL));

  // determine if the title already contains year, this is often the case with
  // movies produced several times
  bool show_year = true;
  string::size_type first_paran = m.m_strTitle.find('(');
  if (first_paran != string::npos) {
    string::size_type second_paran = m.m_strTitle.find(')');
    if (second_paran != string::npos)
      if (second_paran - first_paran == 5)
	show_year = false;
  }

  string title = m.m_strTitle;
  if (show_year)
    title += "(" + conv::itos(m.m_iYear) + ")";
	
  string_format::format_to_size(title, "Vera/25", conf->p_v_res()-2*45, true);
  render->current.add(new TObj(title, "Vera/25", 45, 15, 
			       themes->movie_header_font1, themes->movie_header_font2, 
			       themes->movie_header_font3, 3));

  int y = 80;
  int x = 45;

  PObj *p;

  int w = 100, h = 140;

  if (!file_exists(render->image_get_fullpath(m.cover_path(imdb_dir), w, h))) {
    render->create_scaled_image_wrapper(m.cover_path(imdb_dir), w, h);
    // to insert the image into the transmap so that it won't fail when
    // we draw the screen again
    bool tmp;
    render->image_dimensions(render->image_get_fullpath(m.cover_path(imdb_dir), w, h), 
			     &tmp);
  }

  if (file_exists(m.cover_path(imdb_dir)))
    p = new PFObj(m.cover_path(imdb_dir), x, y, w, h, false, 3);
  else {
    if (conf->p_h_res() == 405)
      p = new PObj(themes->movie_mcover_small, x, y, 3, NOSCALING);
    else
      p = new PObj(themes->movie_mcover, x, y, 3, NOSCALING);
  }

  render->current.add(p);
  
  render->current.add(new TObj(gettext("Director: ") + m.m_strDirector, "Vera/17", x + p->w + 10, y, 
			       themes->movie_font1, themes->movie_font2, themes->movie_font3, 3));
  y+=25;

  render->current.add(new TObj(gettext("Writing Credits: ") + m.m_strWritingCredits, "Vera/17", x + p->w + 10, y, 
			       themes->movie_font1, themes->movie_font2, themes->movie_font3, 3));
  y+=50;

  string runtime = gettext("Runtime: ") + string_format::trim(m.m_strRuntime);
  string_format::format_to_size(runtime, "Vera/17", conf->p_v_res()-(45 + (x + p->w + 10)), true);

  render->current.add(new TObj(runtime, "Vera/17", x + p->w + 10, y, 
			       themes->movie_font1, themes->movie_font2, themes->movie_font3, 3));

  y+=(p->h+15-50-25);

  string genre = gettext("Genre: ");
  bool first = false;
  for (std::vector<std::string>::const_iterator i = m.genres.begin(), iend = m.genres.end(); i != iend; ++i) {
    if (first)
      genre += " /";
    else
      first = true;
    genre += ' ' + *i;
  }
  string_format::format_to_size(genre, "Vera/17", conf->p_v_res()-2*45, true);
  render->current.add(new TObj(genre, "Vera/17", x, y, 
			       themes->movie_font1, themes->movie_font2, 
			       themes->movie_font3, 3));

  y+=2*25;

  int max_lines = 8;

  if (conf->p_v_res() == 800)
    max_lines = 6;
  else if (conf->p_v_res() == 720)
    max_lines = 4;

  int pos = 0, lines = 0;

  string tagline = string_format::trim(m.m_strTagLine);

  if (!tagline.empty()) {
    tagline = gettext("Tagline: ") + tagline;

    while (pos != tagline.size()) {

      if (lines == max_lines)
	break;

      string t = string_format::get_line(tagline, pos, conf->p_v_res()-100, "Vera/17");

      if (lines == max_lines - 1 && pos != tagline.size()) {
	t = t.substr(0, t.size()-3);
	t += "...";
      }
      
      render->current.add(new TObj(t, "Vera/17", x, y + lines*25, 
				   themes->movie_font1, themes->movie_font2, 
				   themes->movie_font3, 3));
      ++lines;
    }
    y+=(lines+1)*25;
  }

  if (conf->p_h_res() == 405)
    y -= 7;

  string tmp_plot = string_format::trim(m.m_strPlot);

  int plot_pos = tmp_plot.find('\n');

  if (plot_pos != string::npos)
    tmp_plot = tmp_plot.substr(0, plot_pos);
  
  string plot = gettext("Plot: ") + string_format::trim(tmp_plot);

  int tag_lines = lines;
  pos = 0, lines = 0;

  while (pos != plot.size() && tag_lines + lines != max_lines) {

    string t = string_format::get_line(plot, pos, conf->p_v_res()-100, "Vera/17");

    if (tag_lines + lines == max_lines - 1 && pos != plot.size()) {
      t = t.substr(0, t.size()-3);
      t += "...";
    }

    render->current.add(new TObj(t, "Vera/17", x, y + lines*25, 
				 themes->movie_font1, themes->movie_font2, 
				 themes->movie_font3, 3));
    ++lines;
  }

  y+=(lines+1)*25;

  render->current.add(new TObj(gettext("Rating:"), 
			       "Vera/17", x, y, themes->movie_font1, themes->movie_font2, 
			       themes->movie_font3, 3));

  int x_size = string_format::calculate_string_width(gettext("Rating:"), "Vera/17");

  int nr_stars = int(m.m_fRating);
  bool half_star = true;
  if (m.m_fRating - nr_stars == 0)
    half_star = false;

  for (int t = 0; t < nr_stars; ++t)
    render->current.add(new PObj(themes->movie_rating_icon, x+x_size+t*30, y, 3, NOSCALING));

  float extra_space = 0.5;
  if (half_star) {
    render->current.add(new PObj(themes->movie_rating_icon_half, x+x_size+nr_stars*30, y, 3, NOSCALING));
    extra_space += 0.7;
  }

  render->current.add(new TObj(conv::ftos(m.m_fRating) + " (" + m.m_strVotes + " votes)", 
			       "Vera/17", int(x+x_size+(nr_stars+extra_space)*30), y, 
			       themes->movie_font1, themes->movie_font2, themes->movie_font3, 3));

  

  y+=25;

  render->draw_and_release("Movie information 1");

  input_master->device->get_input();

  render->prepare_new_image();

  render->current.add(new PObj(themes->movie_background, 0, 0, 0, SCALE_FULL));

  render->current.add(new TObj(title, "Vera/25", 45, 15, 
			       themes->movie_header_font1, themes->movie_header_font2, 
			       themes->movie_header_font3, 3));

  y = 90;
  x = 45;

  render->current.add(new TObj(gettext("Cast: "), "Vera/17", x, y, 
			       themes->movie_font1, themes->movie_font2, 
			       themes->movie_font3, 3));
  y += 25;

  int actors = 0;
  int max_actors = 20;
  if (conf->p_v_res() == 800)
    max_actors = 16;
  else if (conf->p_v_res() == 720)
    max_actors = 14;

  int max_x = 0;
  for (vector<std::pair<string, string> >::const_iterator i = m.actors.begin(), end = m.actors.end(); 
       i != end && actors < max_actors; ++i, ++actors)
    max_x = std::max(max_x, string_format::calculate_string_width(i->first, "Vera/17"));

  actors = 0;

  if (max_x > conf->p_v_res()-(2*45+30))
    max_x = conf->p_v_res()-(2*45+30)/2;

  for (vector<std::pair<string, string> >::const_iterator i = m.actors.begin(), end = m.actors.end(); 
       i != end && actors < max_actors; ++i, ++actors) {
    string cast_member = i->first;
    string_format::format_to_size(cast_member, "Vera/17", max_x, true);
    render->current.add(new TObj(cast_member, "Vera/17", x+30, y, 
				 themes->movie_font1, themes->movie_font2, 
				 themes->movie_font3, 3));
    
    string cast_role = " as   " + i->second;
    string_format::format_to_size(cast_role, "Vera/17", conf->p_v_res()-(2*45+30)-(max_x+1), true);
    render->current.add(new TObj(cast_role, "Vera/17", x+max_x+1+30, y, 
				 themes->movie_font1, themes->movie_font2, 
				 themes->movie_font3, 3));

    y += 25;
  }

  render->draw_and_release("Movie information 2");

  input_master->device->get_input();
}

void MovieDB::imdb_download_movie(CIMDBMovie &m, const CIMDBUrl& url)
{
  BusyIndicator::get_instance().busy();

  Overlay overlay_details = Overlay("dialog");
  DialogStepPrint pdialog2(gettext("Getting details about movie"), 
			   overlay_details);

  imdb.GetDetails(url, m);

  if (m.name.empty()) {
    m.name = m.m_strTitle;
    m.lowercase_name = string_format::lowercase(m.name);
  }

  // fixme: gif?
  string cover = imdb_dir + m.name + "_cover.jpg";
  // delete any previous cover, the new movie might not have a cover
  if (file_exists(cover))
    run::external_program("rm \"" + cover + "\"");

  if (imdb.Download(m.m_strPictureURL, cover)) {
    render->create_scaled_image_wrapper(m.cover_path(imdb_dir), image_width, image_height);
    // to insert the image into the transmap so that it won't fail when
    // we draw the screen again
    bool tmp;
    render->image_dimensions(render->image_get_fullpath(m.cover_path(imdb_dir),
							image_width, 
							image_height), 
			     &tmp);
  }

  pdialog2.print_step2();
}

bool MovieDB::imdb_search_mainloop(const string& search_word, CIMDBMovie &m, bool search)
{
  ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();
  BusyIndicator &busy_indicator = BusyIndicator::get_instance();
  screen_updater->timer.deactivate("audio_fullscreen");
  
  input_master->save_map();
  input_master->set_map("default");

  Input input;

  int pos = -1;
  IMDB_MOVIELIST results;

  if (!search_word.empty()) {
    Overlay overlay = Overlay("dialog");

    DialogStepPrint pdialog(gettext("Looking up movie in IMDb database"), overlay);

    busy_indicator.busy();
    
    imdb.FindMovie(search_word, results);
    if (results.size() > 0)
      pos = 0;
  
    pdialog.print_step2();
  }
  
  // set to idle not matter if the search word is empty or not
  busy_indicator.idle();

  Overlay o = Overlay("imdb search");

  bool found_match = false;

  string search_string = search_word;

  std::list<Input> exit_inputs;
  Input i;
  i.command = "back"; i.mode = "general";
  exit_inputs.push_back(i);

  while (!(input.command == "back" && input.mode == "general"))
    {
      if (pos == -1) {
	MovieIMDbFunctor<MovieDB> print_func(this, &MovieDB::imdb_search_screen, results, pos, o);

	string gen = search_string;
	if (input_master->device->generate_string<MovieIMDbFunctor<MovieDB> >(gen, &print_func, exit_inputs).first)
	  search_string = gen;
	else {
	  input.command = "back";
	  input.mode = "general";
	  continue;
	}

	Overlay overlay_lookup = Overlay("dialog");
	DialogStepPrint pdialog2(gettext("Looking up movie in IMDb database"), 
				 overlay_lookup);

	busy_indicator.busy();

	imdb.FindMovie(search_string, results);
	if (results.size() > 0)
	  pos = 0;

	busy_indicator.idle();

	pdialog2.print_step2();

	continue;

      } else
	imdb_search_screen(search_string, results, pos, o);

      input = input_master->device->get_input();

      if (input.command == "prev")
	{
	  if (pos != -1)
	    --pos;
 	}
      else if (input.command == "next")
 	{
	  if (pos != results.size()-1)
	    ++pos;
 	}
      else if (input.command == "info")
	{
	  render->wait_and_aquire();
	  o.cleanup();
	  render->image_mut.leaveMutex();

	  // download movie and show it
	  CIMDBMovie tmp;
	  imdb_download_movie(tmp, vector_lookup(results, pos));
	  print_information(tmp);

	  o = Overlay("imdb search");
	  MovieIMDbFunctor<MovieDB> print_func(this, &MovieDB::imdb_search_screen, results, pos, o);
	}
      else if (input.command == "action")
	{
	  imdb_download_movie(m, vector_lookup(results, pos));

	  m.Save(physical, db);

	  found_match = true;
	  break;
	}
      global->check_commands(input, true);
    }

  render->wait_and_aquire();
  o.cleanup();
  render->image_mut.leaveMutex();

  screen_updater->timer.activate("audio_fullscreen");
  
  input_master->restore_map();

  return found_match;
}

void MovieDB::imdb_search_screen(string search_word, IMDB_MOVIELIST results, int pos, Overlay& o)
{
  // pos ranges from [-1, results.size()-1], -1 denotes the enter search word box

  render->wait_and_aquire();

  if (o.elements.size() > 0)
    o.partial_cleanup();

  int x, y, max_x = conf->p_v_res()-250, max_y = 30;

  int max = std::min(10, int(results.size()));
  if (max == 0)
    max = 1; // for the no results found mesage

  int box_height = max_y*max+30+35+30+30;

  x = (conf->p_v_res()-(max_x+30))/2;
  y = (conf->p_h_res()-(box_height))/2;

  o.add(new RObj(x-15+2, y-15+2, max_x+30+1, box_height+1, 0, 0, 0, 200, 0));
  o.add(new RObj(x-15, y-15, max_x+30, box_height,  
		 themes->dialog_background1, themes->dialog_background2, 
		 themes->dialog_background3, themes->dialog_background4, 1));

  string header = gettext("IMDb lookup");

  int x_size = string_format::calculate_string_width(header, "Vera/20");

  o.add(new TObj(header, "Vera/20", (conf->p_v_res()-x_size)/2, y-5, 
		 themes->dialog_font1, themes->dialog_font2, themes->dialog_font3, 3));
  y += 35;

  if (pos == -1) 
    o.add(new RObj(x-5, y-3, max_x+10, 25+6, 0, 0, 0, 215, 2));

  string s = gettext("Search: ");
  x_size = string_format::calculate_string_width(s, "Vera/17");
  string_format::format_to_size(search_word, "Vera/17", max_x-x_size, true);

  o.add(new TObj(s + search_word, "Vera/17", x, y, themes->dialog_font1, 
		 themes->dialog_font2, themes->dialog_font3, 3));
  y += 40;

  if (results.size() == 0)
    o.add(new TObj(gettext("No results found..."), "Vera/17", x, y, 
		   themes->dialog_font1, themes->dialog_font2, themes->dialog_font3, 3));
  else {

    int nr_elements = 10;
    int i, start;
    i = std::max(0, pos-nr_elements+1);
    start = i;

    for (IMDB_MOVIELIST::const_iterator iter = results.begin()+i;
	 iter != results.end() && i < (start + nr_elements); ++iter, ++i) {
      if (i == pos)
	o.add(new RObj(x-5, y-3, max_x+10, 25+6, themes->dialog_marked1, 
		       themes->dialog_marked2, themes->dialog_marked3, 
		       themes->dialog_marked4, 2));

      string title = iter->m_strTitle;
      string_format::format_to_size(title, "Vera/17", max_x, true);
      o.add(new TObj(title, "Vera/17", x, y, themes->dialog_font1, 
		     themes->dialog_font2, themes->dialog_font3, 3));
      y += 30;
    }

    string result_string = conv::itos(results.size());

    if (results.size() == 1)
      result_string += gettext(" result found");
    else
      result_string += gettext(" results found");

    x_size = string_format::calculate_string_width(result_string, "Vera/17");

    o.add(new TObj(result_string, "Vera/17", x+max_x-x_size, y+5, themes->dialog_font1, 
		   themes->dialog_font2, themes->dialog_font3, 3));
  }
  
  render->draw_and_release("imdb search screen", true);
}

std::string MovieDB::get_name_from_file(const CIMDBMovie& m)
{
  string name = m.name;
  if (!m.real_name.empty())
    name = m.real_name;
  else if (m.m_strTitle != "")
    name = m.m_strTitle;

  return name;
}

bool MovieDB::create_thumbnail(const std::string& file, const std::string& thumbnail)
{
  run::external_program("mplayer -noconsolecontrols -nojoystick -nolirc -nomouseinput -ao null -vo null -identify -endpos 0 '" + file + "'| grep ID_LENGTH > " + conf->p_temp_image_loc() + "/movie_length");

  std::ifstream In((conf->p_temp_image_loc() + "/movie_length").c_str());
  string str;
  string::size_type pos = 0;
  string length = "";
  while (getline(In, str)) {
    if ((pos = str.rfind("ID_LENGTH=")) != string::npos) {
      length = str.substr(10);
      break;
    }
  }

  if (length.empty())
    return false;

  run::external_program(string("mplayer -noconsolecontrols -nojoystick -nolirc -nomouseinput -ao null -vf scale=") + conv::itos(image_width) + ":-2 -frames 2 -vo jpeg:outdir=" + conf->p_temp_image_loc() + " -ss " + conv::itos(conv::atoi(length)/2) + " '" + file + "' > /dev/null");

  string thumbnail_dir = thumbnail.substr(0, thumbnail.rfind('/')+1);
  create_if_not_exists(thumbnail_dir);

  // FIXME: escape ' so that thumbnails can be created correctly
  run::external_program("mv " + conf->p_temp_image_loc() + "/00000002.jpg '" + thumbnail + "'");

  return true;
}

void MovieDB::print(const vector<CIMDBMovie>& cur_files)
{
  render->prepare_new_image();

  render->current.add(new PObj(themes->movie_background, 0, 0, 0, SCALE_FULL));
  
  render->current.add(new PObj(themes->startmenu_movie_dir, 25, 10, 2, NOSCALING));
  
  if (themes->show_header) {
    string message;

    if (physical)
      message = gettext("Movies");
    else
      message = gettext("Movie Collection");
    
    render->current.add(new TObj(message, "Vera/28", 100, 10, 
				 themes->movie_header_font1, themes->movie_header_font2, 
				 themes->movie_header_font3, 2));
  }
  
  if (search_mode) {

    int y = 60;
    if (conf->p_v_res() == 720) {
      y -= 3;
    }

    render->current.add(new RObj(0, y, conf->p_v_res(), 45, 
				 themes->general_search_rectangle_color1, 
				 themes->general_search_rectangle_color2, 
				 themes->general_search_rectangle_color3, 
				 themes->general_search_rectangle_color4, 2));

    render->current.add(new PObj(themes->general_search, 47, y, 3, NOSCALING));

    render->current.add(new TObj(gettext("Search: ") + search_str, "Vera/22", 100, y+5, 
				 themes->movie_font1, themes->movie_font2, 
				 themes->movie_font3, 3));
  }

  // print the range
  int pos, start, pos_in_list, cur_rows, cur_image_height_all;

  if (search_mode) {
    pos_in_list = offset;
    cur_rows = rows_search;
    cur_image_height_all = image_height_all_search;
  } else {
    pos_in_list = folders.top().second;
    cur_rows = rows;
    cur_image_height_all = image_height_all;
  }

  int nr_in_last_row = cur_files.size()%images_per_row;
  if (nr_in_last_row == 0)
    nr_in_last_row = images_per_row;

  if (cur_files.size() > cur_rows*images_per_row) {
    if (pos_in_list < std::ceil(double(cur_rows)/2.0)*images_per_row) {
      pos = pos_in_list;
      start = 0;
    } else if (pos_in_list >= cur_files.size() - (images_per_row+nr_in_last_row)) { // last two rows
      start = cur_files.size()-((cur_rows-1)*images_per_row+nr_in_last_row);
      pos = pos_in_list-start;
    } else {
      pos = pos_in_list%images_per_row+images_per_row;
      if (pos == images_per_row-1)
	pos = 2*images_per_row-1;
      start = pos_in_list-pos;
    }
  } else {
    pos = pos_in_list;
    start = 0;
  }

  int x_start, x = (conf->p_v_res()-images_per_row*image_width_all)/2, y = y_start;
  x_start = x;
  
  if (search_mode) {
    y = y_start;
    if (conf->p_input() != "keyboard")
      y += 60;
  }

  for (int j = 0; j < cur_rows*images_per_row && j < cur_files.size(); ++j) {

    CIMDBMovie s = vector_lookup(cur_files, start);

    string cover_path = s.cover_path(imdb_dir);

    if (file_exists(cover_path)) {
      bool border = false;
      
      if (pos == j)
	border = true;

      if (!file_exists(render->image_get_fullpath(s.cover_path(imdb_dir), image_width, image_height))) {
	render->create_scaled_image_wrapper(cover_path, image_width, image_height);
	// to insert the image into the transmap so that it won't fail when
	// we draw the screen again
	bool tmp;
	render->image_dimensions(render->image_get_fullpath(cover_path, image_width, image_height), &tmp);
      }

      PFObj *p = new PFObj(cover_path, x, y, image_width, image_height, border, 3);
	
      p->x = p->real_x = x + (image_width_all - (p->w + 10))/2;
      p->y = p->real_y = y + (image_height_all_eks_text - (p->h + 10))/2;

      if (p->border) {
	p->real_x -= 5;
	p->real_y -= 5;
      } else {
	p->real_x -= 1;
	p->real_y -= 1;
      }

      render->current.add(p);

    } else {

      if (s.type != "file") {

	PObj *p;
      
	if (pos == j)
	  p = new PObj(themes->movie_dir_marked, x, y, 3, NOSCALING);
	else
	  p = new PObj(themes->movie_dir, x, y, 3, NOSCALING);

	p->x = p->real_x = x + (image_width_all - p->w)/2;
	p->y = p->real_y = y + (image_height_all_eks_text - p->h)/2;
      
	render->current.add(p);

      } else {

	string thumbnail = "";
  
	string::size_type dot_pos = s.filenames.front().rfind(".");
	if (dot_pos != string::npos)
	  thumbnail = conf->p_temp_image_loc() + s.filenames.front().substr(0, dot_pos) + "_thumb.jpg";

	bool thumbnail_ok = false;

	// try thumbnail
	if (!file_exists(thumbnail))
	  thumbnail_ok = create_thumbnail(s.filenames.front(), thumbnail);
	else
	  thumbnail_ok = true;

	PObj *p;
	if (thumbnail_ok)
	  p = new PObj(thumbnail, x-5, y-5, 3, NOSCALING, false);
	else {
	  if (conf->p_h_res() == 405)
	    p = new PObj(themes->movie_mcover_small, x, y, 3, NOSCALING);
	  else
	    p = new PObj(themes->movie_mcover, x-5, y-5, 3, NOSCALING);
	}
	
	p->x = p->real_x = x + (image_width_all - (p->w + 10))/2;
	p->y = p->real_y = y + (image_height_all_eks_text - (p->h + 10))/2;
	render->current.add(p);

      }
    }

    string name = get_name_from_file(s);

    string old_name = name;
    
    string_format::format_to_size(name, "Vera/13", image_width_all-10, false);

    string name_second_part = old_name.substr(name.size());

    if (name_second_part.size() > 0 && name_second_part[0] != ' ') {
      // splitting words
      string::size_type last_whole_word = name.rfind(' ');
      if (last_whole_word != string::npos) {
	name = name.substr(0, last_whole_word);
	name_second_part = old_name.substr(name.size());
      }
    }

    int x_size = string_format::calculate_string_width(name, "Vera/13");

    string_format::format_to_size(name_second_part, "Vera/13", image_width_all-10, false);

    int x_size2 = string_format::calculate_string_width(name_second_part, "Vera/13");

    if (pos == j) {
      render->current.add(new TObj(name, "Vera/13", x + (image_width_all-x_size)/2, y+image_height_all_eks_text, 
				   themes->pictures_marked_font1, themes->pictures_marked_font2, 
				   themes->pictures_marked_font3, 3));
      render->current.add(new TObj(name_second_part, "Vera/13", x + (image_width_all-x_size2)/2, y+image_height_all_eks_text+20, 
				   themes->pictures_marked_font1, themes->pictures_marked_font2, 
				   themes->pictures_marked_font3, 3));
    } else {
      render->current.add(new TObj(name, "Vera/13", x + (image_width_all-x_size)/2, y+image_height_all_eks_text, 
				   themes->pictures_font1, themes->pictures_font2, 
				   themes->pictures_font3, 3));
      render->current.add(new TObj(name_second_part, "Vera/13", x + (image_width_all-x_size2)/2, y+image_height_all_eks_text+20, 
				   themes->pictures_font1, themes->pictures_font2, 
				   themes->pictures_font3, 3));
    }

    x += image_width_all;

    if (start == cur_files.size()-1) {
      if ((j+1)%images_per_row != 0)
	j += (images_per_row-(j+1)%images_per_row);
      
      x = x_start;
      y += cur_image_height_all;

      start = 0;
      continue;
      
    } else
      ++start;
    
    if ((j+1)%images_per_row == 0) {
      y += image_height_all;
      x = x_start;
    }
  }

  std::ostringstream position;

  if (cur_files.size() > 0) {
    if (search_mode) {
      if (cur_files.size() > 0)
	position << offset + 1 << "/" << cur_files.size();
      else
	position << "";
    } else
      position << folders.top().second+1 << "/" << cur_files.size();
  
    int x_size = string_format::calculate_string_width(position.str(), "Vera/18");
    
    render->current.add(new TObj(position.str(), "Vera/18", conf->p_v_res()-(60+x_size), 20, 
				 themes->movie_font1, themes->movie_font2, 
				 themes->movie_font3, 3));
  }

  render->draw_and_release("movie-graphical");
}
