#include <Python.h>
#include "structmember.h"
#include "pycontrol.hpp"

using namespace pymms::gui;

#ifdef __cplusplus
extern "C" {
#endif

namespace pymms {

namespace python {

static PyObject* ListControl_New(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
  ListControl *self;
  int x = 0, y = 0, w = 0, h = 0, l = 4, tos = 0, itemheight = 30, itemspace = 0;
  char *focus = "";
  char *nofocus = "";
  char *font = "Vera";
  char *rgb = "0xffffff";
  char *rgbfocus = "0xffffff";
  char *alignment = "left";
  
  static char *kwlist[] = {"x", "y", "width", "height", 
                           "font", "rgb", "rgbfocus", "focus", "nofocus", 
			   "textoffset", "alignment", "itemheight", "itemspace", NULL};
  
  self = (ListControl*)type->tp_alloc(type, 0);
  if (!self) 
    return NULL;
    
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "llll|ssssslsll", kwlist, &x, &y, 
                                   &w, &h, &font, &rgb, &rgbfocus, &focus, &nofocus, &tos, &alignment, &itemheight, &itemspace))
    return NULL;
    
  self->guicontrol = new GUIListControl(x, y, w, h, l, focus, nofocus, font, rgb, rgbfocus, tos, alignment, itemheight, itemspace);

  return (PyObject*)self;
}

static PyObject* ListControl_set_label(ListControl *self, PyObject *args, PyObject *kwds)
{
  char *font = NULL;
  char *rgb = NULL;
  char *rgbfocus = NULL;

  static char *kwlist[] = {"font", "rgb", "rgbfocus", NULL};
    
  if (!PyArg_ParseTupleAndKeywords(args, kwds, "|sss", kwlist, &font, &rgb, &rgbfocus))	
    return NULL;

  if(font)
    ((GUIListControl*)self->guicontrol)->set_labelfont(font);
  if(rgb)
    ((GUIListControl*)self->guicontrol)->set_labelrgb(rgb);
  if(rgbfocus)
    ((GUIListControl*)self->guicontrol)->set_labelrgbfocus(rgbfocus);

  Py_INCREF(Py_None);
  return Py_None;
}

static void ListControl_Dealloc(ListControl* self)
{
  for (std::vector<ListItem*>::iterator i = self->items.begin(),
         end = self->items.end(); i != end; i++) {

    Py_DECREF(*i);
  }

  self->guicontrol->clear();
  self->items.clear();

  delete self->guicontrol;
  self->ob_type->tp_free((PyObject*)self);
}

static PyObject* ListControl_add_item(ListControl *self, PyObject *args)
{
  ListItem* item;

  if (!PyArg_ParseTuple(args, "O", &item))
    return NULL;

  if(!ListItem_Check(item))
  {
    PyErr_SetString(PyExc_TypeError, "Object should be of type ListItem");
    return NULL;
  }

  if(!((GUIListControl*)self->guicontrol)->add_item(item->guilistitem))
  {
    PyErr_SetString(PyExc_ReferenceError, "Item is already in list");
    return NULL;
  }

  Py_INCREF(item);

  self->items.push_back(item);

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* ListControl_set_itemheight(ListControl *self, PyObject *args)
{
  int itemheight = 0;

  if (!PyArg_ParseTuple(args, "l", &itemheight))
    return NULL;

  ((GUIListControl*)self->guicontrol)->set_itemheight(itemheight);

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* ListControl_set_itemspace(ListControl *self, PyObject *args)
{
  int itemspace = 0;

  if (!PyArg_ParseTuple(args, "l", &itemspace))
    return NULL;

  ((GUIListControl*)self->guicontrol)->set_itemspace(itemspace);

  Py_INCREF(Py_None);
  return Py_None;
}

PyObject* ListControl_clear(ListControl *self, PyObject *args)
{
  for (std::vector<ListItem*>::iterator i = self->items.begin(),
         end = self->items.end(); i != end; i++) {

    Py_DECREF(*i);
  }

  self->guicontrol->clear();
  self->items.clear();

  Py_INCREF(Py_None);
  return Py_None;
}

PyObject* ListControl_get_selected_position(ListControl *self, PyObject *args)
{
  return PyLong_FromLong(((GUIListControl*)self->guicontrol)->get_selected_position());
}

PyObject* ListControl_get_selected_item(ListControl *self, PyObject *args)
{
  GUIListItem *guiitem;
  ListItem *item = 0;

  if(!(guiitem = ((GUIListControl*)self->guicontrol)->get_selected_item()))
  {
    PyErr_SetString(PyExc_RuntimeError, "No items in list");
    return NULL;
  }

  for (std::vector<ListItem*>::iterator i = self->items.begin(),
      end = self->items.end(); i != end; i++) {
    if((*i)->guilistitem == guiitem)
      item = *i;
  }

  if(!item)
  {
    PyErr_SetString(PyExc_RuntimeError, "This error should not happen :-)");
    return NULL;
  }

  Py_INCREF(item);

  return (PyObject*)item;
}

static PyMethodDef ListControl_methods[] = {
  {"set_label", (PyCFunction)ListControl_set_label, METH_VARARGS | METH_KEYWORDS, 0},
  {"add_item", (PyCFunction)ListControl_add_item, METH_VARARGS, 0},
  {"clear", (PyCFunction)ListControl_clear, METH_VARARGS, 0},
  {"get_selected_item", (PyCFunction)ListControl_get_selected_item, METH_VARARGS, 0},
  {"get_selected_position", (PyCFunction)ListControl_get_selected_position, METH_VARARGS, 0},
  {"set_itemheight", (PyCFunction)ListControl_set_itemheight, METH_VARARGS, 0},
  {"set_itemspace", (PyCFunction)ListControl_set_itemspace, METH_VARARGS, 0},
  {NULL, NULL, 0, NULL}
};

PyTypeObject ListControl_Type = {
	PyObject_HEAD_INIT(NULL)
	0,                         /*ob_size*/
	"mmsv2gui.ListControl",    /*tp_name*/
	sizeof(ListControl),      /*tp_basicsize*/
	0,                         /*tp_itemsize*/
	(destructor)ListControl_Dealloc,/*tp_dealloc*/
	0,                         /*tp_print*/
	0,                         /*tp_getattr*/
	0,                         /*tp_setattr*/
	0,                         /*tp_compare*/
	0,                         /*tp_repr*/
	0,                         /*tp_as_number*/
	0,                         /*tp_as_sequence*/
	0,                         /*tp_as_mapping*/
	0,                         /*tp_hash */
	0,                         /*tp_call*/
	0,                         /*tp_str*/
	0,                         /*tp_getattro*/
	0,                         /*tp_setattro*/
	0,                         /*tp_as_buffer*/
	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
	0,       /* tp_doc */
	0,		                     /* tp_traverse */
	0,		                     /* tp_clear */
	0,		                     /* tp_richcompare */
	0,		                     /* tp_weaklistoffset */
	0,		                     /* tp_iter */
	0,		                     /* tp_iternext */
	ListControl_methods,      /* tp_methods */
	0,                         /* tp_members */
	0,                         /* tp_getset */
	&Control_Type,             /* tp_base */
	0,                         /* tp_dict */
	0,                         /* tp_descr_get */
	0,                         /* tp_descr_set */
	0,                         /* tp_dictoffset */
	0,                         /* tp_init */
	0,                         /* tp_alloc */
	ListControl_New,          /* tp_new */
};

}

}

#ifdef __cplusplus
}
#endif
