#ifndef INPUT_HPP
#define INPUT_HPP

#include "config.h"

#include "common.hpp"
#include "config.hpp"

#include "singleton.hpp"

#include <cc++/thread.h>

#include <string>
#include <map>
#include <vector>

#include <iostream>

// structure used for input keys
class Input
{
public:
  std::string mode, key, command, name;
  bool multiple;

  bool operator==(const Input& i) const
  {
    return mode == i.mode && command == i.command;
  }

  std::string to_string() const;
  // create from string
  Input(std::string& s);

  Input();
};

typedef std::map<std::string, Input> input_map;
typedef std::vector<Input> input_list;
typedef std::map<std::string, std::pair<input_map, input_list> > name_input_map;

class InputDevice;

class InputMaster
{
private:

  std::pair<bool, std::pair<input_map, input_list> > parse_config(const std::string& plugin, 
								  const std::string& version, bool extends);
  bool check_version(const std::string& input, const std::string& str, const std::string& version);

  std::string saved_map, cur_map;
  
public:

  std::string find_shortcut(const std::string& command);
  input_list cur_list();

  name_input_map maps;

  bool restore_map_empty();
  void save_map();
  void restore_map();
  void set_map(const std::string& name);
  std::string current_map();

  bool find_input(const std::string& input, Input &i);

  InputDevice *device;

  // const input_desc_map& desc, 
  bool parse_keys(const std::string& plugin, const std::string& version);
  bool extend_default(const std::string& plugin, const std::string& version);

  InputMaster();
};

typedef Singleton<InputMaster> S_InputMaster;

class GenSearchBaseFunc
{
public:
  virtual ~GenSearchBaseFunc() {}

  virtual int size(const std::string& search_word) = 0;
  virtual std::vector<std::string> vec(const std::string& search_word) = 0;
};

template <typename T1, typename T2, typename TClass>
class GenSearch : public GenSearchBaseFunc
{
 private:
  T1 the_list;
  T2 rep;

  int (TClass::*fpt)(T1, const std::string&, T2);
  std::vector<std::string> (TClass::*fpt2)(T1, const std::string&, T2);
  TClass* pt2Object;

 public:
  
  GenSearch(TClass* _pt2Object, int(TClass::*_fpt)(T1, const std::string&, T2), 
	    std::vector<std::string>(TClass::*_fpt2)(T1, const std::string&, T2), 
	    T1 l, T2 r)
    :  the_list(l), rep(r), fpt(_fpt), fpt2(_fpt2), pt2Object(_pt2Object)
  {}

  int size(const std::string& search_word)
  { return (*pt2Object.*fpt)(the_list, search_word, rep); }

  std::vector<std::string> vec(const std::string& search_word)
  { return (*pt2Object.*fpt2)(the_list, search_word, rep); }
};

template <typename T>
class GenGraphicalSearch : public GenSearchBaseFunc
{
 private:
  T *func;

 public:
  
  GenGraphicalSearch(T *f)
    : func(f)
  {}

  int size(const std::string& search_word)
  { return func->call(search_word).size(); }

  std::vector<std::string> vec(const std::string& search_word)
  { 
    std::vector<std::string> result;
    std::vector<std::pair<std::string, int> > temp = func->call(search_word); 
    for (std::vector<std::pair<std::string, int> >::iterator i = temp.begin(), iend = temp.end(); 
	 i != iend; ++i)
      result.push_back(i->first);
    return result;
  }
};

// used as superclass of input devices
class InputDevice : public ost::Thread
{
public:

  template<typename F>
  std::pair<bool, Input> generate_string(std::string& str, F* print_func, std::list<Input> exit_inputs)
  {
    InputMaster *input_master = S_InputMaster::get_instance();
    Config *conf = S_Config::get_instance();

    input_master->save_map();
    input_master->set_map("search");

    Input device_input;

    bool save_playlist = false;

    enable_all_input();

    generate_search_top();

    while (true) {

      bool value_found = false;
      for (std::list<Input>::const_iterator i  = exit_inputs.begin(), iend = exit_inputs.end();
	   i != iend; ++i)
	if (*i == device_input) {
	  value_found = true;
	  break;
	}
      
      if (value_found)
	break;

      print_func->call(str);

      device_input = get_input();

      if (device_input.command == "action") {
	save_playlist = true;
	break;
      } else {
	generate_search_input(str, str, device_input);
      }
    }
	  
    disable_all_input();

    input_master->restore_map();

    return std::make_pair(save_playlist, device_input);
  }

  // returns db id
  template<typename T, typename F, typename K>
  int graphical_search(const std::vector<T>& current_files, F* print_func, K* gen_list,
		       bool& search_mode, std::string& search_str, std::string& lowercase_search_str, 
		       int& offset, int& search_help_offset)
  {
    InputMaster *input_master = S_InputMaster::get_instance();
    Config *conf = S_Config::get_instance();

    input_master->save_map();
    input_master->set_map("search");

    // reset
    search_str = "";
    lowercase_search_str = "";
    offset = 0;
    search_help_offset = -1;

    search_mode = true;

    Input device_input;

    int pos = -1;

    enable_all_input();

    search_top();

    while (device_input.command != "back") {

      std::vector<std::pair<std::string, int> > db_results = gen_list->call(search_str);

      std::vector<T> results;

      for (typename std::vector<T>::const_iterator j = current_files.begin(), 
	     jend = current_files.end(); j != jend; ++j) {
	for (std::vector<std::pair<std::string, int> >::iterator i = db_results.begin(), 
	       iend = db_results.end(); i != iend; ++i)
	  if (j->db_id == i->second) {
	    results.push_back(*j);
	    break;
	  } 
      }

      print_func->call(results);

      device_input = get_input();

      if (device_input.command == "action") {
	T result = vector_lookup(results, offset % results.size());
	int i = 0;
	for (typename std::vector<T>::const_iterator f = current_files.begin(), 
	       fend = current_files.end(); f != fend; ++i, ++f)
	  if (*f == result)
	    pos = i;
	break;
      } else {
	GenGraphicalSearch<K> gs(gen_list);
	search_graphical_input(search_str, lowercase_search_str,
			       device_input, offset, search_help_offset, &gs);
      }

      trigger_input(input);
    }

    disable_all_input();

    input_master->restore_map();

    search_mode = false;

    return pos;
  }

  template<typename T, typename F, typename K, typename L>
  void search(const std::vector<T>& list, int *position, 
	      F* print_func, K* compare, L* representation,
	      bool& search_mode, std::string& search_str, 
	      std::string& lowercase_search_str, int& offset)
  {
    InputMaster *input_master = S_InputMaster::get_instance();
    Config *conf = S_Config::get_instance();

    input_master->save_map();
    input_master->set_map("search");

    search_str = "";
    lowercase_search_str = "";
    offset = 0;
    search_mode = true;

    Input device_input;

    bool canceled = true;

    enable_all_input();

    search_top();

    while (device_input.command != "back") {

      print_func->call(gen_search(list, compare));

      device_input = get_input();

      if (device_input.command == "action") {
	canceled = false;
	break;
      } else {
	GenSearch<const std::vector<T>&, L*, InputDevice> gs(this, &InputDevice::gen_searched_list_size,
							     &InputDevice::gen_searched_list, list, representation);
	search_input(search_str, lowercase_search_str, device_input, offset, &gs);
      }
    }

    disable_all_input();

    // find position in the original list
    if (!canceled) {

      std::vector<std::string> searched = gen_searched_list(list, search_str, representation);

      if (searched.size() > 0) {
	std::string element = vector_lookup(searched, offset%searched.size());

	for (int i = 0, size = list.size(); i < size; ++i)
	  if (representation->call(vector_lookup(list, i)) == element) {
	    *position = i;
	    break;
	  }
      }
    }

    input_master->restore_map();

    search_mode = false;
  }

  void wait_for_input_grab()
  {
    input_done_.wait();
  }

  // helper function
  Input get_input();

  // for searching
  void enable_all_input() { accept_all = true; }
  void disable_all_input() { accept_all = false; }

  virtual void suspend() = 0;
  virtual void wake_up() = 0;

  virtual bool init() = 0;

  virtual ~InputDevice() {};
  InputDevice();

protected:

  void trigger_input(Input input);

  virtual void generate_search_top() = 0;
  virtual void generate_search_input(std::string& str, std::string& lstr, Input input) = 0;

  virtual void search_top() = 0;
  virtual void search_input(std::string& str, std::string& lowercase_search_str, 
			    Input input, int& offset, GenSearchBaseFunc *func) = 0;
  virtual void search_graphical_input(std::string& str, std::string& lowercase_search_str, Input input, 
				      int& offset, int& search_help_offset, GenSearchBaseFunc *func) = 0;

  void complete_string(std::string &search_str, std::string &lowercase_search_str, GenSearchBaseFunc *func)
  {
    std::vector<std::string> searched = func->vec(search_str);

    if (searched.size() != 0) {
      if (searched.size() == 1) {
	search_str = vector_lookup(searched, 0);
	lowercase_search_str = string_format::lowercase(search_str);
      } else {
	std::string gcss = string_format::greatest_common_substring(string_format::lowercase(vector_lookup(searched, 0)),
								    string_format::lowercase(vector_lookup(searched, 1)));
	for (int i = 2; i < searched.size(); ++i)
	  gcss = string_format::greatest_common_substring(gcss, string_format::lowercase(vector_lookup(searched, i)));

	search_str = gcss;
	lowercase_search_str = string_format::lowercase(search_str);
      }
    }
  } 

  bool accept_all;

  Input input;

  ost::Event input_ready;
  ost::Event input_done_;

  template<typename T, typename F>
  const std::vector<T> gen_search(const std::vector<T>& list, F* compare)
  {
    std::vector<T> newlist;

    for (int i = 0, size = list.size(); i < size; ++i)
      if (compare->call(vector_lookup(list, i))) 
	newlist.push_back(vector_lookup(list, i));

    return newlist;
  }

  template<typename T, typename L>
  std::vector<std::string> gen_searched_list(const std::vector<T>& list, const std::string& search_word, L* representation)
  {
    // optimization
    if (search_word.empty()) {
      std::vector<std::string> tmp;
      for (int i = 0, size = list.size(); i < size; ++i)
	tmp.push_back(representation->call(vector_lookup(list, i)));
      return tmp;
    }
    
    std::vector<std::string> newlist;

    std::string lowercase_search_str = string_format::lowercase(search_word);

    for (int i = 0, size = list.size(); i < size; ++i)
      if (string_format::lowercase(representation->call(vector_lookup(list, i))).substr(0, search_word.size())
	  == lowercase_search_str)
	newlist.push_back(representation->call(vector_lookup(list, i)));
    
    return newlist;
  }

  template<typename T, typename L>
  int gen_searched_list_size(const std::vector<T>& list, const std::string& search_word, L* representation)
  {
    // optimization
    if (search_word.empty())
      return 0;

    int amount = 0;

    std::string lowercase_search_str = string_format::lowercase(search_word);

    for (int i = 0, size = list.size(); i < size; ++i)
      if (string_format::lowercase(representation->call(vector_lookup(list, i))).substr(0, search_word.size())
	  == lowercase_search_str)
	++amount;
    
    return amount;
  }

private:

  void register_keypress(Input input);

  virtual void run() = 0;
};

#endif
